/* main.c - Application main entry point */

/*
 * Copyright (c) 2015-2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/types.h>
#include <stddef.h>
#include <sys/printk.h>
#include <sys/util.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>

static uint8_t mfg_data[] = { 0xff, 0xff, 0x00, 0x00 };

static const struct bt_data ad[] = {
	BT_DATA(BT_DATA_MANUFACTURER_DATA, mfg_data, 4),
};


struct trigger_adv {
	void *reserved;
	bool extended;
};

static struct k_poll_signal tx_done = K_POLL_SIGNAL_INITIALIZER(tx_done);

K_FIFO_DEFINE(adv_fifo);

static void bt_adv_done(struct bt_le_ext_adv *adv,
			struct bt_le_ext_adv_sent_info *info)
{
	ARG_UNUSED(adv);
	ARG_UNUSED(info);

	k_poll_signal_raise(&tx_done, 0);
}

void main(void)
{
	struct bt_le_adv_param *adv_param = BT_LE_ADV_PARAM(BT_LE_ADV_OPT_NONE, BT_GAP_ADV_FAST_INT_MIN_1, BT_GAP_ADV_FAST_INT_MAX_1, NULL);
	struct bt_le_ext_adv *adv;
	struct bt_le_ext_adv_cb adv_cb = {
		.sent = bt_adv_done
	};
	struct k_poll_event tx_done_cb = K_POLL_EVENT_INITIALIZER(
		K_POLL_TYPE_SIGNAL, K_POLL_MODE_NOTIFY_ONLY, &tx_done);
	int err;

	printk("Starting Fault Demo\n");

	/* Initialize the Bluetooth Subsystem */
	err = bt_enable(NULL);
	if (err) {
		printk("Bluetooth init failed (err %d)\n", err);
		return;
	}

	printk("Bluetooth initialized\n");

	/* Create a non-connectable non-scannable advertising set */
	err = bt_le_ext_adv_create(adv_param, &adv_cb, &adv);
	if (err) {
		printk("Failed to create advertising set (err %d)\n", err);
		return;
	}

	struct trigger_adv *to_do;

	do {
		to_do = k_fifo_get(&adv_fifo, K_FOREVER);

		/* Set correct options */
		if (to_do->extended) {
			adv_param->options =  BT_LE_ADV_OPT_EXT_ADV;
		} else {
			adv_param->options = BT_LE_ADV_OPT_NONE;
		}

		err = bt_le_ext_adv_update_param(adv, adv_param);
		if (err) {
			printk("Failed to set advertising parameters (err %d)\n", err);
			return;
		}

		err = bt_le_ext_adv_set_data(adv, ad, ARRAY_SIZE(ad), NULL, 0);
		if (err) {
			printk("Failed to set advertising data (err %d)\n", err);
			return;
		}

		k_poll_signal_reset(&tx_done);
		err = bt_le_ext_adv_start(adv, BT_LE_EXT_ADV_START_PARAM(0, 1));
		if (err) {
			printk("Advertising failed to start (err %d)\n", err);
			return;
		}
		mfg_data[2]++;

		/* Wait for transmission to complete */
		err = k_poll(&tx_done_cb, 1, K_SECONDS(1));
		if (err < 0) {
			printk("Advertising failed to complete\n");
			return;
		}
	} while (1);
}

void thread1(void)
{
	struct trigger_adv s = { .extended = true };

	for (;;) {
		k_fifo_put(&adv_fifo, &s);
		k_sleep(K_MSEC(1000));
	}
}

K_THREAD_DEFINE(t1, 2048, thread1, NULL, NULL, NULL, 5, 0, 0);

void thread2(void)
{
	struct trigger_adv s = { .extended = false };


	for (;;) {
		k_fifo_put(&adv_fifo, &s);
		k_sleep(K_MSEC(5000));
	}
}

K_THREAD_DEFINE(t2, 2048, thread2, NULL, NULL, NULL, 5, 0, 0);
